#!/bin/bash
# Takes as input the name of a Geolocation HDF5 file
# Creates a file named DayNightCheck.txt in the current directory
# The file contains one line of the form
# 	DAYNIGHT="XXX"
# where XXX is Day if the input file contains at least one day granule,
# otherwise Night.

# Clean any existing files
rm -f tmpa.txt
rm -f tmpb.txt
rm -f DayNightCheck.txt

granulepath=`${H5UTILHOME}/h5dump -A $1 |grep -A 1 'All_Data' |grep -v 'All_Data' | grep  -Po '".*?"' |sed  's/"//g'`
null=`${H5UTILHOME}/h5dump  -d  /All_Data/$granulepath/ModeGran -o tmpa.txt  -y $1`

cat tmpa.txt | sed -e 's/[\t ]//g;/^$/d' > tmpb.txt
cat tmpb.txt | sed s/,/\\n/g  > tmpa.txt

numOfDayGran=0
numOfNightGran=0
numOfBothGran=0
totalnumOfGran=0

while read line; 
do 
dnbval=$line
if [ $dnbval -eq 1 ]; then
let numOfDayGran=numOfDayGran+1 
elif [ $dnbval -eq 0 ]; then
let numOfNightGran=numOfNightGran+1 
elif [ $dnbval -eq 2 ]; then
let numOfBothGran=numOfBothGran+1 
fi
let totalnumOfGran=totalnumOfGran+1
done < tmpa.txt

# Threshold defines the minimum number of day granules that are needed to mark it as a Day Pass
# For a single granule we set the threshold as 1 else we set the threshold to atleast 2 granule
# We do this because we do not want to miss a case where we have a single granule and it is a day pass.
if [ $totalnumOfGran -le 1 ]; then
threshold=1
else 
threshold=2
fi

if [ $numOfDayGran -ge $threshold ]; then
  echo DAYNIGHT=\"Day\" > DayNightCheck.txt
else 
  echo DAYNIGHT=\"Night\" > DayNightCheck.txt
fi