#!/bin/bash
# Script that runs VFIRE375 2.4.2 given an input and output directory
# Usage:
#	run_VFIRE375.sh <input directory> <output directory>

# First, find out where we are
case $0 in
         /*)  SHELLFILE=$0 ;;
		./*)  SHELLFILE=${PWD}${0#.} ;;
        ../*) SHELLFILE=${PWD%/*}${0#..} ;;
          *)  SHELLFILE=$(type -P "$0") ; if [ "${SHELLFILE:0:1}" != "/" ]; then SHELLFILE="${PWD}/$SHELLFILE" ; fi ;;
esac

# readlink -m makes sure SHELLDIR is a simple absolute path
SHELLDIR=`readlink -m "${SHELLFILE%/*}"`

###############################################################
# !!! UPDATE THIS AS NECESSARY FOR EVERY NEW RELEASE !!!
###############################################################
# VFIRE375 directory is one directory above:
ALGODIR="$SHELLDIR/../vfire375-2.5.1"

###############################################################

# Make the input/output directories absolute paths
INPUTDIR=`readlink -m "$1"`
OUTPUTDIR=`readlink -m "$2"`

# Find the VIAES from the input directory; complain if not found
VIAES_REL=`find $1 -name "NPP_VIAES_L1*" -print -quit`
VIAES_ABS=`readlink -m "$VIAES_REL"`

if [ -z "$VIAES_REL" ]; then
	echo "ERROR - VIAES product not found"
	exit 1
fi

# Find the IMFTS from the input directory; complain if not found
IMFTS_REL=`find $1 -name "NPP_IMFTS_L1*" -print -quit`
IMFTS_ABS=`readlink -m "$IMFTS_REL"`

if [ -z "$IMFTS_REL" ]; then
	echo "ERROR - IMFTS product not found"
	exit 1
fi

# Find the VMAES from the input directory; complain if not found
VMAES_REL=`find $1 -name "NPP_VMAES_L1*" -print -quit`
VMAES_ABS=`readlink -m "$VMAES_REL"`

if [ -z "$VMAES_REL" ]; then
	echo "ERROR - VMAES product not found"
	exit 1
fi

# Find the VCDGIPS from the input directory; complain if not found
VCDGIPS_REL=`find $1 -name "NPP_VCDGIPS_L1*" -print -quit`
VCDGIPS_ABS=`readlink -m "$VCDGIPS_REL"`

if [ -z "$VCDGIPS_REL" ]; then
	echo "ERROR - VCDGIPS product not found"
	exit 1
fi

# Find the QSLWMIP from the input directory; complain if not found
QSLWMIP_REL=`find $1 -name "NPP_QSLWMIP_L1*" -print -quit`
QSLWMIP_ABS=`readlink -m "$QSLWMIP_REL"`

if [ -z "$QSLWMIP_REL" ]; then
	echo "ERROR - QSLWMIP product not found"
	exit 1
fi

# With all products found, run the VFIRE375 algorithm, storing outputs in the output directory
eval "$ALGODIR/vfire375 -a $OUTPUTDIR/vfire375FLOC.txt -dv $VIAES_ABS $IMFTS_ABS $VMAES_ABS $VCDGIPS_ABS $QSLWMIP_ABS $INPUTDIR/reference.txt $OUTPUTDIR/vfire375FMASK.hdf"
