#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <math.h>


#include "misc.h"
#include "error.h"
#include "const.h"
#include "types.h"



int Detect_Daytime(Juggernaut *, FireData *fire_data);
int Postscreen_Daytime(Juggernaut *, FireData *fire_data);

/* Run tests on all daytime pixels */
/* Populate jugg->pixflags */
/* Populate some FireData fields */
int Detect_Daytime(Juggernaut *jugg, FireData *fire_data){
	int i, j, x, y, left, right, k;
	int z, d, lw_flag;
	int target_pixel = 0;

	float T4, T5, Dif45;
	int R1, R2, R3, Ref12;

	int good_pixels = 0;
	int bf_pixels = 0;
	float scene_T4[251001];
	float scene_T5[251001];
	float scene_Dif45[251001];
	float mbf_T4[251001];
	float scene_median;
	float scene_back;
	float median_T4;
	float *ptr_scene_T4;
	int window_size;
	float area;
	int startx, endx;

	float dist2mean_T4[251001];
	float dist2mean_T5[251001];
	float dist2mean_Dif45[251001];

	float mean_T4;
	float mean_T5;
	float mean_Dif45;
	float mean_bf_T4;

	float meandev_T4;
        float meandev_T5;
        float meandev_Dif45;
	float meandev_bf_T4;

	/* flags */
	int height;
	int flag = 0;
	int top, bottom;

	int bf_mask_count;
	int water_mask_count;
	int mask_count;
	long total_pixels = 0;

	/* Run over all pixels */
	for(i=0;i<jugg->number_of_scan_lines;i++){
		for(j=0;j<SAMPLES_375M;j++){

			/* Must be clear land and daytime */
			if(jugg->pixflags[i][j].mask != 5) continue;
			if(jugg->pixflags[i][j].night != 0) continue;

			/* Reset varibles */
			good_pixels = 0;
			median_T4 = 0;

			/* Easier to use variables */
			R1 = jugg->R1[i][j];
			R2 = jugg->R2[i][j];
			R3 = jugg->R3[i][j];
			T4 = jugg->T4[i][j];
			T5 = jugg->T5[i][j];
			Ref12 = R1 + R2;
			Dif45 = T4 - T5;
		
					
			/* Skip bright pixels (flase alarms along the mississippi river banks) */
			if(R3 > 3000 && R3 > R2 && R2 > 2500 && T4 <= 335){
				/* Meta data 9 */
				jugg->pixflags[i][j].meta_data |= 0x200;
				continue;
			}

			/* Skip non-candidate fire pixels */
			if(T4 <= 325 || Dif45 <= 25) continue;

			/* Meta data 10 */
			jugg->pixflags[i][j].meta_data |= 0x400;

			/* Flag fire pixel over glint */
			if( (jugg->pixflags[i][j].glint_level == 2 && Ref12 > 3500) || (jugg->pixflags[i][j].glint_level == 1 && Ref12 > 4000) ){
				jugg->pixflags[i][j].mask = 2;
				continue;
			}

			/* Create 500 by 500 window */
			/* Left bound */
			left = MAX(j - 250, 0);

			/* Right bound */
			right = MIN(j + 250, SAMPLES_375M-1);

			/* Top bound */
			top = MAX(i-250, 0);

			/* Bottom bound */
			bottom = MIN(i + 250, jugg->number_of_scan_lines-1);

			for(x=top;x<=bottom;x++){
				for(y=left;y<=right;y++){
					if(x == i && y == j) continue;
					if(jugg->pixflags[x][y].mask == 5 && jugg->pixflags[x][y].bf_mask == 0 && jugg->pixflags[x][y].water_mask == 0 && jugg->pixflags[x][y].night == 0){
						scene_T4[good_pixels] = jugg->T4[x][y];
						good_pixels++;
					}
				}
			}

			/* Not enough good pixels */
			if (good_pixels < 10){
				jugg->pixflags[i][j].mask = 6;
				continue;
			}


			/* Get T4 median value  */
			ptr_scene_T4 = scene_T4; // can be done with just passing the address (&scene_T4)
			median_T4 = median(ptr_scene_T4, good_pixels);						
		
			/* Find scene back (bt4)  */
			scene_back = MAX(325, (median_T4 + 25));

			/* Doesn't stick out */
			if(T4 <= MIN(330, scene_back)) continue;

			/* Meta data 11 */
			jugg->pixflags[i][j].meta_data |= 0x800;

			/* Use smaller window */
			window_size = 10;
			flag = 0;
			while(flag == 0 && window_size <= 30){
				/* Reset flags */
				good_pixels = bf_pixels = 0;
				flag = 0;
				
				/* --- Get bounds --- */
				left = MAX(j-window_size, 0);
				right = MIN(j+window_size, SAMPLES_375M-1);
				top = MAX(i-window_size, 0);
				bottom = MIN(i+window_size, jugg->number_of_scan_lines-1);
				/* --- --- */



				/* Current segment */
				for(x=top;x<=bottom;x++){
					for(y=left;y<=right;y++){

						if(x == i && y == j){
							target_pixel = 1;
						}else{
							target_pixel = 0;
						}

						if(jugg->LWM[i/2][j/2] == 17 || jugg->LWM[i/2][j/2] == 18){
							lw_flag = 1;
						}else{
							lw_flag = 0;
						}
						if(!target_pixel && jugg->pixflags[x][y].mask == 5 && jugg->pixflags[x][y].bf_mask == 0 && jugg->pixflags[x][y].water_mask == lw_flag && jugg->pixflags[x][y].night == 0){
							scene_T4[good_pixels] = jugg->T4[x][y];
							scene_T5[good_pixels] = jugg->T5[x][y];
							scene_Dif45[good_pixels] = jugg->T4[x][y] - jugg->T5[x][y];
							good_pixels++;
						}
						/* For later rejection of false alarms */
						if(jugg->pixflags[x][y].bf_mask == 1 && jugg->pixflags[x][y].night == 0){
							mbf_T4[bf_pixels] = jugg->T4[x][y];
							bf_pixels++;
						}
					}
				}
			
				/* Check if enough good pixel for area looked at */
				height = bottom - top;
				area = (right-left+1)*(height+1);
				if(good_pixels < 10 || good_pixels < area/4){
					window_size += 5;
				}else{
					flag = 1;
				}
			} /* Small box while end */

			/* Not enough good pixels found using small window */
			if(flag == 0){
				jugg->pixflags[i][j].mask = 6;
				continue;
			}


			/* Get mean */
			mean_T4 = mean(scene_T4, good_pixels);
			mean_T5 = mean(scene_T5, good_pixels);
			mean_Dif45 = mean(scene_Dif45, good_pixels);

			/* Find mean dev */
			for(d=0;d<good_pixels;d++){
				dist2mean_T4[d] = fabsf(scene_T4[d]-mean_T4);
				dist2mean_T5[d] = fabsf(scene_T5[d]-mean_T5);
				dist2mean_Dif45[d] = fabsf(scene_Dif45[d]-mean_Dif45);
			}
			meandev_T4 = mean(dist2mean_T4, good_pixels);
			meandev_T5 = mean(dist2mean_T5, good_pixels);
			meandev_Dif45 = mean(dist2mean_Dif45, good_pixels);


			
			if(bf_pixels){
				/* Get mean */
				mean_bf_T4 = mean(mbf_T4, bf_pixels);
				/* Find mean dev */
				for(d=0;d<bf_pixels;d++){
					dist2mean_T4[d] = fabsf(mbf_T4[d]-mean_bf_T4);
				}
				meandev_bf_T4 = mean(dist2mean_T4, bf_pixels);
	
				/* Mark false alarm  in mask */
				if( (bf_pixels > 0.1*good_pixels) && bf_pixels > 4 && R2 > 1500 && mean_bf_T4 < 345 && meandev_bf_T4 < 3 && (T4 < mean_bf_T4 + meandev_bf_T4*6) ){
					jugg->pixflags[i][j].mask = 5;
					/* old Meta data 18 */
					//jugg->pixflags[i][j].meta_data |= 0x40000; 
					continue;
				}
			}else{
				meandev_bf_T4 = 0;
			}

			/* Meta data 12 */
			if(Dif45 > mean_Dif45+2*meandev_Dif45) jugg->pixflags[i][j].meta_data |= 0x1000;
			/* Meta data 13 */
			if(Dif45 > mean_Dif45+10) jugg->pixflags[i][j].meta_data |= 0x2000;
			/* Meta data 14 */
			if(T4 > mean_T4+3.5F*meandev_T4) jugg->pixflags[i][j].meta_data |= 0x4000;
			/* Meta data 15 */
			if(T5 > mean_T5+meandev_T5-4 || meandev_bf_T4 > 5) jugg->pixflags[i][j].meta_data |= 0x8000;
			
			
			/* Fire found */
			if(Dif45 > mean_Dif45+2*meandev_Dif45 && Dif45 > mean_Dif45+10 && T4 > mean_T4+3.5F*meandev_T4 && (T5 > mean_T5+meandev_T5-4 || meandev_bf_T4 > 5) ){	

				jugg->pixflags[i][j].mask = 8;
				/* Fill in the necessary fire data */
				fire_data->Row[fire_data->index] = i;
				fire_data->Col[fire_data->index] = j;
				fire_data->mean_T4[fire_data->index] = mean_T4;
				fire_data->mean_T5[fire_data->index] = mean_T5;
				fire_data->MAD_T4[fire_data->index] = meandev_T4;
				fire_data->MAD_T5[fire_data->index] = meandev_T5;
				fire_data->mean_Dif45[fire_data->index] = mean_Dif45;
				fire_data->MAD_Dif45[fire_data->index] = meandev_T4 - meandev_T5;
				fire_data->Window_Size[fire_data->index] = window_size;
				fire_data->index++;

				/* Check if fire_data needs to be realloced */
				if(fire_data->index%5000 == 0) FireDataAllocate(fire_data);				
			}


		}
	}

}


/* Run after Detect_Daytime in order to check for low-confidence fire pixels */ 
int Postscreen_Daytime(Juggernaut *jugg, FireData *fire_data){
	int i, j, x, y, k;
	float T4, T5, Dif45;
	int R1, R2, R3, Ref12;
	float minimum;
	int bad1, bad2, bad3, bad4, bad5;
	int bad_scanline_count;

	int fire_flag, glint_count, array_index;
	float array[8];

	/* Check for low-confidence fire pixels */
	for(i=0;i<jugg->number_of_scan_lines;i++){
		for(j=0;j<SAMPLES_375M;j++){
			if(jugg->pixflags[i][j].night != 0) continue;

			fire_flag = 0;

			/* Easier to use variables */
			R1 = jugg->R1[i][j];
			R2 = jugg->R2[i][j];
			R3 = jugg->R3[i][j];
			T4 = jugg->T4[i][j];
			T5 = jugg->T5[i][j];
			Ref12 = R1 + R2;
			Dif45 = T4 - T5;


	
			/* Reworked fire pixel folding and saturation */
			if( (T5 >= 325 || T4 == 367 || Dif45 < 0) && (jugg->pixflags[i][j].mask == 5 || jugg->pixflags[i][j].mask == 0) ){
				/* Meta data 16 */
				jugg->pixflags[i][j].meta_data |= 0x10000;
				/* Check adjacent pixels for fires */
				for(y=-1;y<=1;y++){
					for(x=-1;x<=1;x++){
						if(y == 0 && x == 0) continue;
						if(i+y < 0 || i+y >= jugg->number_of_scan_lines) continue;
						if(j+x < 0 || j+x >= SAMPLES_375M) continue;

						if(jugg->pixflags[i+y][j+x].mask >= 8 && jugg->pixflags[i+y][j+x].mask <= 9 && jugg->pixflags[i+y][j+x].night == 0){
							fire_flag = 1;
						}
						
					}
				}
				
				/* If fire found in adjacent pixel, mark as low-confidence fire */
				if(fire_flag){
					/* Fill in the necessary fire data */
					fire_data->Row[fire_data->index] = i;
					fire_data->Col[fire_data->index] = j;
					fire_data->mean_T4[fire_data->index] = 0;
					fire_data->mean_T5[fire_data->index] = 0;
					fire_data->MAD_T4[fire_data->index] = 0;
					fire_data->MAD_T5[fire_data->index] = 0;
					fire_data->mean_Dif45[fire_data->index] = 0;
					fire_data->MAD_Dif45[fire_data->index] = 0;
					fire_data->Window_Size[fire_data->index] = 0;
					fire_data->index++;
					/* Fill in the necessary fire data */
					if(fire_data->index%5000 == 0) FireDataAllocate(fire_data);
					
					jugg->pixflags[i][j].mask = 7;
				}

			}
		}
	}

	/* Check for low-confidence fire pixels */
	for(i=0;i<jugg->number_of_scan_lines;i++){
		bad_scanline_count = 0;
		for(j=0;j<SAMPLES_375M;j++){
			if(jugg->pixflags[i][j].night != 0) continue;

			glint_count = 0;
			fire_flag = 0;
			array_index = 0; 
			minimum = 10000;

			/* Easier to use variables */
			R1 = jugg->R1[i][j];
			R2 = jugg->R2[i][j];
			R3 = jugg->R3[i][j];
			T4 = jugg->T4[i][j];
			T5 = jugg->T5[i][j];
			Ref12 = R1 + R2;
			Dif45 = T4 - T5;

			/* flag low confidence fire pixels ( potentally glint related) */
			if((Dif45 <= 30 || jugg->pixflags[i][j].glint_level == 2) && jugg->pixflags[i][j].mask == 8){
				
				/* Meta data 17 */
				jugg->pixflags[i][j].meta_data |= 0x20000; 
				/* Check adjacent pixels for fires */
				for(y=-1;y<=1;y++){
					for(x=-1;x<=1;x++){
						if(y == 0 && x == 0) continue;
						if(i+y < 0 || i+y >= jugg->number_of_scan_lines) continue;
						if(j+x < 0 || j+x >= SAMPLES_375M) continue;

						if(jugg->pixflags[i+y][j+x].mask == 2 && jugg->pixflags[i+y][j+x].night == 0){
							glint_count++;
						}
						if(jugg->pixflags[i+y][j+x].mask == 9 && jugg->pixflags[i+y][j+x].night == 0){
							fire_flag = 1;
						}
						if((jugg->pixflags[i+y][j+x].mask == 5 || jugg->pixflags[i+y][j+x].mask == 2) && jugg->pixflags[i+y][j+x].night == 0){
							array[array_index] = jugg->T4[i+y][j+x];
							array_index++;
						}


					}
				}/* End of adjacent loop */

				/* Mark as glint if flagged */
				if(glint_count > 1){
					jugg->pixflags[i][j].mask = 7;
				}else if(fire_flag){
					continue;
				}else if(array_index > 0){
					for(k=0;k<array_index;k++){
						if(T4-array[k] < minimum) minimum = T4-array[k];	
					}
					if(minimum <= 15){
						jugg->pixflags[i][j].mask = 7;
					}
				}else{ // NEW - 01/08/16
					jugg->pixflags[i][j].mask = 7;
				}

			}


		}

	}
	
	/* Final check of water and glint */
	/* (Check if able to add in another loop) */
	for(i=0;i<jugg->number_of_scan_lines;i++){
		for(j=0;j<SAMPLES_375M;j++){

			if(jugg->pixflags[i][j].night != 0) continue;
			
			/* Mark water pixels */
			if(jugg->pixflags[i][j].mask == 5 && jugg->pixflags[i][j].water_mask == 1) jugg->pixflags[i][j].mask = 3;
		
			/* Mark glint */
			if(jugg->pixflags[i][j].mask == 5 && jugg->pixflags[i][j].glint_level == 2) jugg->pixflags[i][j].mask = 2;
		}
	}

}

