

#include "frp.h"

int Calculate_FRP(Juggernaut *jugg, FireData *fire_data){
	int m_scan_lines = jugg->number_of_scan_lines / 2;
	int top, bottom, left, right, count, height, area, flag, abs_sample, f, fires, i, j, x, y, window_size;
	int8 **MFireMask;
	int8 **val_bin;
	int8 **val_bt4;
	float ps, ss, st, ds, dt, pix_area, theta, temp, sqrt_temp, rad13_sum, abs_scan;

	double sigma = 5.6704*pow(10,-8);   //5.6704e-8;    //[W m^-2 K^-4] Stefan-Boltzman constant
	double a = 2.88*pow(10,-9);
	float h = 824.0;
	float earth_radius = 6371.0;
	float r = earth_radius+h;
	float pt = 0.721;
	float scan_rate = 56.06/3152;

	if ( !(MFireMask = (int8**)malloc(m_scan_lines*sizeof(int8*))) )
		return error(ERR_F_MEM);
	if ( !(MFireMask[0] = (int8*)malloc(m_scan_lines*SAMPLES_750M*sizeof(int8))) )
		return error(ERR_F_MEM);
	for (i=1;i<m_scan_lines;i++){
		MFireMask[i] = MFireMask[i-1] + SAMPLES_750M;
	}
	if ( !(val_bt4 = (int8**)malloc(m_scan_lines*sizeof(int8*))) )
		return error(ERR_F_MEM);
	if ( !(val_bt4[0] = (int8*)malloc(m_scan_lines*SAMPLES_750M*sizeof(int8))) )
		return error(ERR_F_MEM);
	for (i=1;i<m_scan_lines;i++){
		val_bt4[i] = val_bt4[i-1] + SAMPLES_750M;
	}
	if ( !(val_bin = (int8**)malloc(m_scan_lines*sizeof(int8*))) )
		return error(ERR_F_MEM);
	if ( !(val_bin[0] = (int8*)malloc(m_scan_lines*SAMPLES_750M*sizeof(int8))) )
		return error(ERR_F_MEM);
	for (i=1;i<m_scan_lines;i++){
		val_bin[i] = val_bin[i-1] + SAMPLES_750M;
	}
	for(i=0;i<m_scan_lines;i++){
		for(j=0;j<SAMPLES_750M;j++){
			MFireMask[i][j] = 0;
			val_bin[i][j] = 0;
			val_bt4[i][j] = 0;
		}
	}
	
	for(i=0;i<jugg->number_of_scan_lines;i++){
		for(j=0;j<SAMPLES_375M;j++){
			if(jugg->pixflags[i][j].mask >= 7 && jugg->pixflags[i][j].mask <= 9){
				MFireMask[i/2][j/2] = 1;
			}
			if( (jugg->SolZenAng[i][j] <= 90 && jugg->T4[i][j] >= 325) || (jugg->SolZenAng[i][j] > 90 && jugg->T4[i][j] >= 295) ){
				val_bt4[i/2][j/2] = 1;
			}
			if(jugg->pixflags[i][j].mask == 5){
				val_bin[i/2][j/2] = 1;
			}
		}
	}
	int debug_i, debug_j;
	
	for(f=0;f<fire_data->index;f++){
		debug_i = fire_data->Row[f];
		debug_j = fire_data->Col[f];
		i = fire_data->Row[f]/2;
		j = fire_data->Col[f]/2;

		window_size = 1;
		flag = 0;
		while(flag == 0 && window_size <= 10){
			/* --- Get bounds --- */
			left = MAX(j-window_size, 0);
			right = MIN(j+window_size, SAMPLES_750M-1);
			top = MAX(i-window_size, 0);
			bottom = MIN(i+window_size, m_scan_lines-1);
			/* --- --- */
			
			count = 0;
			rad13_sum = 0;
			for(x=top;x<=bottom;x++){
				for(y=left;y<=right;y++){
					if (x == i && y == j) continue;
					//	ind_val = WHERE(sub_val EQ 1 AND sub_val_bt4 EQ 0 AND (rad13[xy[0],xy[1]]-sub_rad13 GT 0),count_val)
					if ( val_bin[x][y] == 1 && val_bt4[x][y] == 0 && (jugg->Rad13_agg[i][j]-jugg->Rad13_agg[x][y] > 0) ){
						count++;
						rad13_sum += jugg->Rad13_agg[x][y];
					}
				}
			}

			height = bottom - top;
			area = (right-left+1)*(height+1);
			if(count < 8 || count < (float)area/4.0){
				window_size += 1;
			}else{
				flag = 1;
			}
		}
		if(flag == 0){
			count = 0;
			rad13_sum = 0;
			for(x=top;x<=bottom;x++){
				for(y=left;y<=right;y++){
					if (x == i && y == j) continue;
					// sub_rad13 GT 0 AND sub_val_bt4 EQ 0 AND (rad13[xy[0],xy[1]]-sub_rad13 GT 0)
					if ( jugg->Rad13_agg[x][y] > 0 && val_bt4[x][y] == 0 && (jugg->Rad13_agg[i][j]-jugg->Rad13_agg[x][y] > 0) ){
						count++;
						rad13_sum += jugg->Rad13_agg[x][y];
					}
				}
			}

			if(count == 0){
				rad13_sum = 0;
				for(x=top;x<=bottom;x++){
					for(y=left;y<=right;y++){
						if (x == i && y == j) continue;
						// sub_rad13 GT 0 AND rad13[xy[0],xy[1]]-sub_rad13 GT 0 AND sub_mask EQ 0
						if ( jugg->Rad13_agg[x][y] > 0 && MFireMask[x][y] == 0 && (jugg->Rad13_agg[i][j]-jugg->Rad13_agg[x][y] > 0) ){
							count++;
							rad13_sum += jugg->Rad13_agg[x][y];
						}
					}
				}
			}

			if(count == 0) continue;
						
		}

		abs_sample = abs(j - 1600);

		if(abs_sample <= 592){
			abs_scan = abs_sample * 3 * scan_rate;
			ps = 0.256*3;
		}else if(abs_sample > 592 && abs_sample <= 960){
			abs_scan = (1776+((abs_sample-592)*2))*scan_rate;
			ps = 0.256*2;
		}else{
			abs_scan = (2512+(abs_sample-960))*scan_rate;
			ps = 0.256;
		}
		ss = ps/h;
		st = pt/h;
		theta = abs_scan/(180/M_PI);
		temp = pow(earth_radius/r,2)-pow(sin(theta),2);
		sqrt_temp = sqrt(temp);
		ds = earth_radius*ss*(cos(theta)/sqrt_temp-1.0);
		dt = r*st*(cos(theta)-sqrt_temp);
		pix_area = ds*dt;
		
		fires = 0;
		if(jugg->pixflags[i*2][j*2].mask >= 7 && jugg->pixflags[i*2][j*2].mask <= 9) fires++;
		if(jugg->pixflags[i*2+1][j*2].mask >= 7 && jugg->pixflags[i*2+1][j*2].mask <= 9) fires++;
		if(jugg->pixflags[i*2][j*2+1].mask >= 7 && jugg->pixflags[i*2][j*2+1].mask <= 9) fires++;
		if(jugg->pixflags[i*2+1][j*2+1].mask >= 7 && jugg->pixflags[i*2+1][j*2+1].mask <= 9) fires++;

		fire_data->FRP[f] = (pix_area*sigma/a)*(jugg->Rad13_agg[i][j]-(rad13_sum/count))/fires;
		fire_data->Rad13[f] = jugg->Rad13_agg[i][j];
		fire_data->mean_Rad13[f] = (rad13_sum/count);
	}


	free(MFireMask[0]);
	free(MFireMask);

	free(val_bt4[0]);
	free(val_bt4);

	free(val_bin[0]);
	free(val_bin);
	
}
