/* 
	Copyright © 1999-2007, United States Government as represented by the 
	Administrator for The National Aeronautics and Space Administration.
	All Rights Reserved.
*/
#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <math.h>
#include <unistd.h>


#include "misc.h"
#include "error.h"
#include "const.h"
#include "types.h"
#include "input.h"


	/*
	;Run lw_filter
	ind_fire_water = WHERE(mask GE 7 AND mask LE 9 AND ((lwmask GE 17 AND lwmask LE 19) OR lwmask EQ 252),count_lw_filter)

	IF (count_lw_filter GT 0) THEN BEGIN

 	
		;Will try and build on the input data used for the sa_filter (if applicable) 
		IF count_sa GT 0 THEN 
			lw_filter,mask,lwmask,bt13_agg,ref_w,lat,lon,midtime,ind_fire_water 
		ELSE BEGIN
			read_hdf4_viirs_radm13,vmae_file,m13_rad_agg,qa_m13,dims_m13_rad
			read_hdf4_viirs_unag,vcdgip_file,'6',m13_rad_unagg,dims_m13
			build_agg_bt13_max,m13_rad_unagg,m13_rad_agg,bt13_agg

			lw_filter,mask,lwmask,bt13_agg,ref_w,lat,lon,midtime,ind_fire_water
		ENDELSE


	;Make a copy of the ref_w to help with testing (to be removed later on)
	FILE_COPY,ref_w,STRCOMPRESS(STRMID(ref_w,0,STRLEN(ref_w)-4)+'_'+time_str+'.txt',/REMOVE_ALL)
	ENDIF   
	*/

int LW_Filter(Juggernaut *jugg, InputInfo *in_info, StaticFireReference *txt_fire_ref, StaticFireReference *mem_fire_ref){
	int i,j,x,y,m,temp_int;
	long time_threshold, count_persist;
	double grid_x, grid_y;
	int flag, adjacent_flag, txt_entry_flag, mem_entry_flag;
	float max_T13;
	long delta_t = 30*86400*pow(10,6);

	ReadRefFile(jugg, in_info, txt_fire_ref);

	/* */
	if ( !(mem_fire_ref->MidTimes = (long**)malloc(5000*sizeof(long*))) )
		return error(ERR_F_MEM);
	if ( !(mem_fire_ref->MidTimes[0] = (long*)malloc(5000*4*sizeof(long))) )
		return error(ERR_F_MEM);
	for (i=1;i<5000;i++){
		mem_fire_ref->MidTimes[i] = mem_fire_ref->MidTimes[i-1] + 4;
	}

	/* */
	if ( !(mem_fire_ref->Grid_XY = (float**)malloc(5000*sizeof(float*))) )
		return error(ERR_F_MEM);
	if ( !(mem_fire_ref->Grid_XY[0] = (float*)malloc(5000*2*sizeof(float))) )
		return error(ERR_F_MEM);
	for (i=1;i<5000;i++){
		mem_fire_ref->Grid_XY[i] = mem_fire_ref->Grid_XY[i-1] + 2;
	}
	
	int8 **mask_back;
	if ( !(mask_back = (int8**)malloc(jugg->number_of_scan_lines/2*sizeof(int8*))) )
		return error(ERR_F_MEM);
	if ( !(mask_back[0] = (int8*)malloc(jugg->number_of_scan_lines/2*SAMPLES_750M*sizeof(int8))) )
		return error(ERR_F_MEM);
	for (i=1;i<(jugg->number_of_scan_lines/2);i++){
		mask_back[i] = mask_back[i-1] + SAMPLES_750M;
	}

	/* zero mask_back (M-band size) */
	for(i=0;i<jugg->number_of_scan_lines/2;i++){
		for(j=0;j<SAMPLES_750M;j++){
			mask_back[i][j] = 0;
		}
	}
	
	/* set mask_back (M-band size) */
	for(i=0;i<jugg->number_of_scan_lines;i++){
		for(j=0;j<SAMPLES_375M;j++){
			if(jugg->pixflags[i][j].mask >= 7 || jugg->pixflags[i][j].mask == 4 || jugg->LWM[i/2][j/2] < 17){
				mask_back[i/2][j/2] = 1;
			}
		}
	}


	count_persist = 0;
	for(i=0;i<jugg->number_of_scan_lines;i++){
		for(j=0;j<SAMPLES_375M;j++){
			if (jugg->pixflags[i][j].mask >= 7 && jugg->pixflags[i][j].mask <= 9 && ((jugg->LWM[i/2][j/2] >= 17 && jugg->LWM[i/2][j/2] <= 19) || jugg->LWM[i/2][j/2] == 252)){
				if(jugg->LWM[i/2][j/2] == 252){
					jugg->pixflags[i][j].mask = 1;
					continue;
				}
				/* Meta data 19 */
				jugg->pixflags[i][j].meta_data |= 0x80000;

				/* Grid Coords */
				if (jugg->lon[i][j] >= 0){
					grid_x = (double)((long)(jugg->lon[i][j]/0.004)*0.004+0.002);
				}else{
					grid_x = (double)((long)(jugg->lon[i][j]/0.004)*0.004-0.002);
				}
				if (jugg->lat[i][j] >= 0){
					grid_y = (double)((long)(jugg->lat[i][j]/0.004)*0.004+0.002);
				}else{
					grid_y = (double)((long)(jugg->lat[i][j]/0.004)*0.004-0.002);
				}

				/* Find adjacent pixels */
				max_T13 = -1;
				adjacent_flag = 0;
				for(y=-1;y<=1;y++){
					for(x=-1;x<=1;x++){
						// DRL: Checks were fixed to reflect actual equations used to calculate indices, and also
						// I-band limits were changed to M-band limits because the arrays in question have M-band
						// dimensions:
						//if(i+y < 0 || i+y >= jugg->number_of_scan_lines) continue;
						//if(j+x < 0 || j+x >= SAMPLES_375M) continue;
						if( ((int)(i/2+y)) < 0 || ((int)(i/2+y)) >= ((int)(jugg->number_of_scan_lines/2)) ) continue;
						if( ((int)(j/2+x)) < 0 || ((int)(j/2+x)) >= SAMPLES_750M ) continue;
						
						if(mask_back[i/2+y][j/2+x] == 0 && jugg->T13[i/2+y][j/2+x] > 0){
							adjacent_flag = 1;
							if(jugg->T13[i/2+y][j/2+x] > max_T13) max_T13 = jugg->T13[i/2+y][j/2+x];
						}
					}
				}

				if (adjacent_flag == 1){
					/* Test if BT13 is less than 3K above background - if so, will assign unique class to flag potential anomaly */
					if ((jugg->T13[i/2][j/2]-max_T13) < 2.5){
						/* Meta data 20 */
						jugg->pixflags[i][j].meta_data |= 0x100000;
						flag = 3;
					}else{
						flag = jugg->pixflags[i][j].mask;
					}
				}else{
					flag = 6;
				}

				if(txt_fire_ref->entries == 0){
					mem_entry_flag = 0;
					for(m=0;m<mem_fire_ref->entries;m++){
						if( fabs(grid_x - mem_fire_ref->Grid_XY[m][0]) < 0.001 && fabs(grid_y - mem_fire_ref->Grid_XY[m][1]) < 0.001 ){
							mem_fire_ref->MidTimes[m][3] = (long) jugg->MidTime[i/32];
							jugg->pixflags[i][j].mask = flag;
							mem_entry_flag = 1;
							break;
						}
					}
					
					if(mem_entry_flag == 0){
						mem_fire_ref->Grid_XY[mem_fire_ref->entries][0] = grid_x;
						mem_fire_ref->Grid_XY[mem_fire_ref->entries][1] = grid_y;
						mem_fire_ref->MidTimes[mem_fire_ref->entries][0] = 0;
						mem_fire_ref->MidTimes[mem_fire_ref->entries][1] = 0;
						mem_fire_ref->MidTimes[mem_fire_ref->entries][2] = 0;
						mem_fire_ref->MidTimes[mem_fire_ref->entries][3] = (long) jugg->MidTime[i/32];
						mem_fire_ref->entries++;
						jugg->pixflags[i][j].mask = flag;
					}
					
				}else{
					txt_entry_flag = 0;
					
					for(m=0;m<txt_fire_ref->entries;m++){
						if( fabs(grid_x - txt_fire_ref->Grid_XY[m][0]) < 0.001 && fabs(grid_y - txt_fire_ref->Grid_XY[m][1]) < 0.001 ){
							/* check if all 3 older points are within the last month... if so, count_persist - else mask = flag */
							// DRL: Cast MidTime as a long first?
							//time_threshold = jugg->MidTime[i/32]-delta_t;
							time_threshold = ((long)jugg->MidTime[i/32]) - delta_t;

							if(txt_fire_ref->MidTimes[m][0] >= time_threshold && txt_fire_ref->MidTimes[m][1] >= time_threshold && txt_fire_ref->MidTimes[m][2] >= time_threshold){
								count_persist++; /* pointless? */
							}else{
								/* Meta data 21 */
								jugg->pixflags[i][j].meta_data |= 0x200000;
								jugg->pixflags[i][j].mask = flag;	
							}
							/* Added to protect running old data */
							if(txt_fire_ref->MidTimes[m][2] > (long) jugg->MidTime[i/32]){
								txt_fire_ref->MidTimes[m][3] = 0;
							}else{
								txt_fire_ref->MidTimes[m][3] = (long) jugg->MidTime[i/32];
							}
							txt_entry_flag = 1;
							break;
						}
					}
					if (txt_entry_flag == 0){
						mem_entry_flag = 0;
						for(m=0;m<mem_fire_ref->entries;m++){
							if( fabs(grid_x - mem_fire_ref->Grid_XY[m][0]) < 0.001 && fabs(grid_y - mem_fire_ref->Grid_XY[m][1]) < 0.001 ){
								mem_fire_ref->MidTimes[m][3] = (long) jugg->MidTime[i/32];
								jugg->pixflags[i][j].mask = flag;
								mem_entry_flag = 1;
								break;
							}
						}
						if(mem_entry_flag == 0){
							mem_fire_ref->Grid_XY[mem_fire_ref->entries][0] = grid_x;
							mem_fire_ref->Grid_XY[mem_fire_ref->entries][1] = grid_y;
							mem_fire_ref->MidTimes[mem_fire_ref->entries][0] = 0;
							mem_fire_ref->MidTimes[mem_fire_ref->entries][1] = 0;
							mem_fire_ref->MidTimes[mem_fire_ref->entries][2] = 0;
							mem_fire_ref->MidTimes[mem_fire_ref->entries][3] = (long) jugg->MidTime[i/32];
							mem_fire_ref->entries++;
							jugg->pixflags[i][j].mask = flag;
						}
					}
				}

			}
		}
	}

	free(mask_back[0]);
	free(mask_back);
	return 0;
}


int ReadRefFile(Juggernaut *jugg, InputInfo *in_info, StaticFireReference *txt_fire_ref){
	char line[150];
	char* token;
	char seps[]   = ", ";
	int i, token_count, line_count, c, lines;
	double temp_double;
	long temp_long;

	/* Open ref file */
	if( access(in_info->infile_ref, F_OK) == 0 ) {
	    in_info->ref_file_ptr = fopen(in_info->infile_ref, "r");
	} else {
	    in_info->ref_file_ptr = fopen(in_info->infile_ref, "w+");
	}
	
	/* Start Lock */
	if (flock(fileno(in_info->ref_file_ptr), LOCK_EX) == -1){
		printf("Error: flock did not work\n");
		exit(-1);
	}

	/* Need to find out number of lines */
	lines = 0;
	while ((c = fgetc(in_info->ref_file_ptr)) != EOF){
		if (c == '\n') ++lines;
	}
	txt_fire_ref->entries = lines;
	fseek(in_info->ref_file_ptr, 0, SEEK_SET);

	/* */
	if ( !(txt_fire_ref->MidTimes = (long**)malloc(txt_fire_ref->entries*sizeof(long*))) )
		return error(ERR_F_MEM);
	if ( !(txt_fire_ref->MidTimes[0] = (long*)malloc(txt_fire_ref->entries*4*sizeof(long))) )
		return error(ERR_F_MEM);
	for (i=1;i<txt_fire_ref->entries;i++){
		txt_fire_ref->MidTimes[i] = txt_fire_ref->MidTimes[i-1] + 4;
	}

	/* */
	if ( !(txt_fire_ref->Grid_XY = (double**)malloc(txt_fire_ref->entries*sizeof(double*))) )
		return error(ERR_F_MEM);
	if ( !(txt_fire_ref->Grid_XY[0] = (double*)malloc(txt_fire_ref->entries*2*sizeof(double))) )
		return error(ERR_F_MEM);
	for (i=1;i<txt_fire_ref->entries;i++){
		txt_fire_ref->Grid_XY[i] = txt_fire_ref->Grid_XY[i-1] + 2;
	}


	line_count = 0;
        while(fgets(line, sizeof line, in_info->ref_file_ptr) != NULL){
		token = strtok (line, seps);
		token_count = 0;
		while (token != NULL){
			if (token_count < 2){
				sscanf(token, "%lf", &temp_double);
				txt_fire_ref->Grid_XY[line_count][token_count] = temp_double;
				token = strtok (NULL, seps);				
			}else{
				sscanf(token, "%ld", &temp_long);
				txt_fire_ref->MidTimes[line_count][token_count-2] = temp_long;
				token = strtok (NULL, seps);
			}
			token_count++;
		}
		txt_fire_ref->MidTimes[line_count][3] = 0;
		line_count++;
	}
	

	/* reopen needed after read so the file can be overwritten */
	in_info->ref_file_ptr = freopen(in_info->infile_ref, "w", in_info->ref_file_ptr);

	return 0;
}
