#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <sys/utsname.h>

#include "misc.h"


/*
This is a bit of a kludge since there seems to be no POSIX
standard for the maximum size of the strings in the utsname
structure.  On some systems the length of each field is SYS_NMLN,
but under Linux and other operating systems it's called
something else.  We'll be safe using the total size of the
structure, however, at the expense of a miniscule amount of wasted
memory.
*/
#define MAX_UNAME_LEN sizeof(struct utsname)


char *make_uname(void)
{
	struct utsname name;
	static char unamestr[MAX_UNAME_LEN];

	(void) uname(&name);

	/* Note: space for the whitespace and terminating null is
	guaranteed because each of the five string fields we're
	extracting has a terminating null.  This guarantees at least
	five unused characters in unamestr[].  If additional printing
	characters are added to the format, however, add the same
	number of characters in the definition of MAX_UNAME_LEN above. */
	sprintf(unamestr, "%s %s %s %s %s", name.sysname, name.nodename,
		name.release, name.version, name.machine);

	return unamestr;
}


#define ELEM_SWAP(a,b) { register float t=(a);(a)=(b);(b)=t; }

float median(float *arr, int n) 
{
    int low, high;
    int median;
    int middle, ll, hh;

    low = 0 ; high = n-1 ; median = (low + high) / 2;
    for (;;) {
        if (high <= low) /* One element only */
            return arr[median] ;

        if (high == low + 1) {  /* Two elements only */
            if (arr[low] > arr[high])
                ELEM_SWAP(arr[low], arr[high]) ;
            return arr[median] ;
        }

    /* Find median of low, middle and high items; swap into position low */
    middle = (low + high) / 2;
    if (arr[middle] > arr[high])    ELEM_SWAP(arr[middle], arr[high]) ;
    if (arr[low] > arr[high])       ELEM_SWAP(arr[low], arr[high]) ;
    if (arr[middle] > arr[low])     ELEM_SWAP(arr[middle], arr[low]) ;

    /* Swap low item (now in position middle) into position (low+1) */
    ELEM_SWAP(arr[middle], arr[low+1]) ;

    /* Nibble from each end towards middle, swapping items when stuck */
    ll = low + 1;
    hh = high;
    for (;;) {
        do ll++; while (arr[low] > arr[ll]) ;
        do hh--; while (arr[hh]  > arr[low]) ;

        if (hh < ll)
        break;

        ELEM_SWAP(arr[ll], arr[hh]) ;
    }

    /* Swap middle item (in position low) back into correct position */
    ELEM_SWAP(arr[low], arr[hh]) ;

    /* Re-set active partition */
    if (hh <= median)
        low = ll;
        if (hh >= median)
        high = hh - 1;
    }
}


float mean(float array[], int count){
	int i;
	float total = 0;
	for(i=0;i<count;i++){
		total += array[i];
	}
	return total/count;
}



