#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <math.h>


#include "misc.h"
#include "error.h"
#include "const.h"
#include "types.h"



void Detect_Nighttime(Juggernaut *jugg, FireData *fire_data);
int Postscreen_Nighttime(Juggernaut *jugg, FireData *fire_data);
void ReplaceWater(Juggernaut *jugg);

void Detect_Nighttime(Juggernaut *jugg, FireData *fire_data){

	int i, j, x, y, left, right;
	int d;
	int target_pixel = 0;
	float T4, T5;
	float Dif45;

	int good_pixels = 0;
	int bf_pixels = 0;
	float scene_T4[251001];
	float scene_T5[251001];
	float scene_Dif45[251001];
	float mbf_T4[251001];
	float median_T4;
	int window_size;
	float area;

	float dist2mean_T4[251001];
	float dist2mean_T5[251001];
	float dist2mean_Dif45[251001];

	float mean_T4;
	float mean_T5;
	float mean_Dif45;

	float meandev_T4;
        float meandev_T5;
        float meandev_Dif45;

	/* flags */
	int prev_flag, next_flag;
	int height;
	int flag = 0;
	int top, bottom;

	/* Run through all pixels */
	for(i=0;i<jugg->number_of_scan_lines;i++){
		for(j=0;j<SAMPLES_375M;j++){
			/* Must be night and clear land */
			if(jugg->pixflags[i][j].night != 1) continue;
			if(jugg->pixflags[i][j].mask != 5) continue;			

			/* Reset varibles */
			good_pixels = 0;
			median_T4 = 0;

			/* Easier to use variables */
			T4 = jugg->T4[i][j];
			T5 = jugg->T5[i][j];
			Dif45 = T4 - T5;

			/* Skip non-candidate fire pixels */
			if(T4 < 295 || Dif45 < 10) continue;

			/* Meta data 10 */
			jugg->pixflags[i][j].meta_data |= 0x400;
			

			/* Use small window */
			window_size = 10;
			flag = 0;
			while(flag == 0 && window_size <= 35){
				/* Reset flags */
				good_pixels = bf_pixels = 0;
				flag = 0;
				
				/* --- Get bounds --- */
				left = MAX(j-window_size, 0);
				right = MIN(j+window_size, SAMPLES_375M-1);
				top = MAX(i-window_size, 0);
				bottom = MIN(i+window_size, jugg->number_of_scan_lines-1);
				/* --- --- */

				
				
				/* 500 window */
				for(x=top;x<=bottom;x++){
					for(y=left;y<=right;y++){
						if(x == i && y == j){
							target_pixel = 1;
						}else{
							target_pixel = 0;
						}
							
						if(!target_pixel && jugg->pixflags[x][y].mask == 5 && jugg->pixflags[x][y].bf_mask == 0 && jugg->pixflags[x][y].water_mask == 0 && jugg->pixflags[x][y].night == 1){
							scene_T4[good_pixels] = jugg->T4[x][y];
							scene_T5[good_pixels] = jugg->T5[x][y];
							
							scene_Dif45[good_pixels] = jugg->T4[x][y] - jugg->T5[x][y];
							good_pixels++;
						}
						/* For later rejection of false alarms */
						if(jugg->pixflags[x][y].bf_mask == 1 && jugg->pixflags[x][y].night == 1){
							mbf_T4[bf_pixels] = jugg->T4[x][y];
							bf_pixels++;
						}
					}
				}

				/* Check if enough good pixel for area looked at */
				height = bottom - top;
				area = (right-left+1)*(height+1);
				if(good_pixels < 10 || good_pixels < area/4){

					window_size += 5;
				}else{
					flag = 1;
				}

			} /* Small box while end */

	


			/* Not enough good pixels found using small window */
			if(flag == 0){
				jugg->pixflags[i][j].mask = 6;
				continue;
			}

			/* Get mean */
			mean_T4 = mean(scene_T4, good_pixels);
			mean_T5 = mean(scene_T5, good_pixels);
			mean_Dif45 = mean(scene_Dif45, good_pixels);

			/* Find mean dev */
			for(d=0;d<good_pixels;d++){
				dist2mean_T4[d] = fabsf(scene_T4[d]-mean_T4);
				dist2mean_T5[d] = fabsf(scene_T5[d]-mean_T5);
				dist2mean_Dif45[d] = fabsf(scene_Dif45[d]-mean_Dif45);
			}
			meandev_T4 = mean(dist2mean_T4, good_pixels);
			meandev_T5 = mean(dist2mean_T5, good_pixels);
			meandev_Dif45 = mean(dist2mean_Dif45, good_pixels);

			/* Meta data 12 */
			if(Dif45 > mean_Dif45+3*meandev_Dif45){
				jugg->pixflags[i][j].meta_data |= 0x1000;
			}

			/* Meta data 13 */
			if(Dif45 > mean_Dif45+9){
				jugg->pixflags[i][j].meta_data |= 0x2000;
			}

			/* Meta data 14 */
			if(T4 > mean_T4+3*meandev_T4){
				jugg->pixflags[i][j].meta_data |= 0x4000;
			}

			/* Fire found */
			if(Dif45 > mean_Dif45+3*meandev_Dif45 && Dif45 > mean_Dif45+9 && T4 > mean_T4+3*meandev_T4){
				jugg->pixflags[i][j].mask = 8;
				
				/* Fill in the necessary fire data */
				fire_data->Row[fire_data->index] = i;
				fire_data->Col[fire_data->index] = j;
				fire_data->mean_T4[fire_data->index] = mean_T4;
				fire_data->mean_T5[fire_data->index] = mean_T5;
				fire_data->MAD_T4[fire_data->index] = meandev_T4;
				fire_data->MAD_T5[fire_data->index] = meandev_T5;
				fire_data->mean_Dif45[fire_data->index] = mean_Dif45;
				fire_data->MAD_Dif45[fire_data->index] = meandev_T4 - meandev_T5;
				fire_data->Window_Size[fire_data->index] = window_size;
				fire_data->index++;
				/* Fill in the necessary fire data */
				if(fire_data->index%5000 == 0) FireDataAllocate(fire_data);
				
			}


		}
	}
	

}



/* Run after Detect_Nighttime */
/* Contains the South American filter */
int Postscreen_Nighttime(Juggernaut *jugg, FireData *fire_data){
	int i, j, x, y, i_inc, j_inc;
	int i_mband, j_mband;
	int adjacent_fires;
	float max_T13;
	int m_scan_lines = jugg->number_of_scan_lines / 2;
	float **MFireMask;

	if ( !(MFireMask = (float**)malloc(m_scan_lines*sizeof(float*))) )
		return error(ERR_F_MEM);
	if ( !(MFireMask[0] = (float*)malloc(m_scan_lines*SAMPLES_750M*sizeof(float))) )
		return error(ERR_F_MEM);
	for (i=1;i<m_scan_lines;i++){
		MFireMask[i] = MFireMask[i-1] + SAMPLES_750M;
	}
	for(i=0;i<m_scan_lines;i++){
		for(j=0;j<SAMPLES_750M;j++){
			MFireMask[i][j] = 0;
		}
	}

	for(i=0;i<jugg->number_of_scan_lines;i++){
		for(j=0;j<SAMPLES_375M;j++){
			if(jugg->pixflags[i][j].mask >= 8 && jugg->pixflags[i][j].mask <= 9){
				MFireMask[i/2][j/2] = 1;
			}
		}
	}

	for(i=0;i<jugg->number_of_scan_lines;i++){
		for(j=0;j<SAMPLES_375M;j++){


			if(jugg->pixflags[i][j].night != 1) continue;
			if(jugg->pixflags[i][j].mask < 8 || jugg->pixflags[i][j].mask > 9) continue;
			if(jugg->lat[i][j] >= 7 || jugg->lat[i][j] <= -55 || jugg->lon[i][j] >= 11 || jugg->lon[i][j] <= -110) continue;

			/* M13 */
			i_mband = i/2;
			j_mband = j/2;

			
			if(jugg->QF13[i_mband][j_mband] != 0) continue;
			if(jugg->T13[i_mband][j_mband] == -1) continue;

			/* Reset variable */
			max_T13 = 0;
			adjacent_fires = 0;



			/* Check adjacent pixels for fires */
			for(y=-1;y<=1;y++){
				for(x=-1;x<=1;x++){
					if(y == 0 && x == 0) continue;
					if(i+y < 0 || i+y >= jugg->number_of_scan_lines) continue;
					if(j+x < 0 || j+x >= SAMPLES_375M) continue;

					if(jugg->pixflags[i+y][j+x].mask >= 8 && jugg->pixflags[i+y][j+x].mask <= 9 && jugg->pixflags[i+y][j+x].night == 1){
						adjacent_fires++;
					}

				}
			}/* End of adjacent loop */


			if(adjacent_fires != 0) continue;

			
			/* Check adjacent for max T13 value */
			for(y=-1;y<=1;y++){
				for(x=-1;x<=1;x++){
					if(y == 0 && x == 0) continue;
					if(i_mband+y < 0 || i_mband+y >= m_scan_lines) continue;
					if(j_mband+x < 0 || j_mband+x >= SAMPLES_750M) continue;

					if(jugg->QF13[i_mband+y][j_mband+x] == 0 && MFireMask[i_mband+y][j_mband+x] == 0 && jugg->T13[i_mband+y][j_mband+x] >  max_T13){
						max_T13 = jugg->T13[i_mband+y][j_mband+x];
					}

				}
			}/* End of adjacent loop */

			/* Mark anomaly */
			if(jugg->T13[i_mband][j_mband]-max_T13 < 2){
				/* Meta data 18 */
				jugg->pixflags[i][j].meta_data |= 0x40000;
				jugg->pixflags[i][j].mask = 5;
			}

			
		}
	}

	free(MFireMask[0]);
	free(MFireMask);

	return 0;
}


void ReplaceWater(Juggernaut *jugg){
	int i, j;

	/* Run through all pixels */
	for(i=0;i<jugg->number_of_scan_lines;i++){
		for(j=0;j<SAMPLES_375M;j++){
			if(jugg->pixflags[i][j].mask != 5) continue;
			if(jugg->LWM[i/2][j/2] >= 17 && jugg->LWM[i/2][j/2] <= 19) jugg->pixflags[i][j].mask = 3;
		}
	}

}





