#ifndef TYPES_H
#define TYPES_H

#include "hdf.h"

typedef unsigned char UBYTE;
typedef unsigned short USHORT;
//typedef unsigned char uint8;
enum Satellite {SAT_NPP, SAT_UNKNOWN};
enum {WATER, COAST, LAND};


/* pixel-level flags and values used internally by algorithm */
typedef struct {
	unsigned day		: 1;	/* daytime = 1, nighttime = 0 */
	unsigned night		: 1;

	unsigned missing_data	: 1;	/* flag: missing radiance data */
	unsigned missing_geo	: 1;	/* flag: bad geolocation */
	unsigned missing_lc	: 1;	/* flag: missing land cover */

							
	unsigned glint_level	: 2;	/* sun glint level (0-2) */

	unsigned meta_data : 32;

	unsigned mask	: 8;	/* mask (0-255) */
	unsigned water_mask	: 1;	/* mask (0-1) */
	unsigned bf_mask	: 1;	/* mask (0-1) */

	} PixInfo;

/* Contains input data, counts, pixflags */
typedef struct {

	float	**T4;
	float	**T5;

	int16	**R1;
	int16	**R2;
	int16	**R3;

	uint8	**QF1;
	uint8	**QF2;
	uint8	**QF3;
	uint8	**QF4;
	uint8	**QF5;

	float	**lat;
	float	**lon;

	float	**SolZenAng;
	float	**SolAzAng;
	float	**ViewZenAng;
	float	**ViewAzAng;
	float64	*MidTime;
	uint8	**QFGeo;

	/* used internally by algorithm */
	PixInfo	**pixflags;

	uint8	**QF13; // from agg input
	float	**T13;
	float	**Rad13_agg;

	uint8	**LWM;

	int	sa_night_pass;

	int	day_count;
	int	night_count;

	int	land_count;
	int	water_count;

	int	missing_count;
	int	glint_count;

	int	cloud_count;

	int	glint_rejection_count;

	int number_of_scan_lines;



	} Juggernaut;


/* Contains info on each fire pixel */
typedef struct{
	int32 index;
	int allocations;

	char Granule[30];
	
	uint8 *Mask; 
	float *Lon;
	float *Lat;
	uint16 *Row;
	uint16 *Col;
	float *T4;
	float *T5;
	uint16 *Window_Size;
	float *SolZenAng;
	float *SolAzAng;
	float *ViewZenAng;
	float *ViewAzAng;
	uint8 *Day;
	float *mean_T4;
	float *mean_T5;
	float *MAD_T4;
	float *MAD_T5;
	float *mean_Dif45;
	float *MAD_Dif45;
	uint16 *Adj_Cloud;
	uint16 *Adj_Water;
	float *FRP;
	float *Rad13;
	float *mean_Rad13;


	} FireData;

/* Contains info on each pixel */
typedef struct{
	int entries;
	double **Grid_XY;
	long **MidTimes;
	uint8 *updated;

	} StaticFireReference;
#endif
