#!/bin/bash -e
# Example preconfigure.sh, for the case where an SPA cares about both
# PROCESSING_MODE and ADDITIONAL_SPA_INSTANCES.
# Takes no arguments - assumes it's running from NISGS_ROOT/SPA/spa_directory

# First find out where we are.

case $0 in
         /*)  SHELLFILE=$0 ;;
        ./*)  SHELLFILE=${PWD}${0#.} ;;
        ../*) SHELLFILE=${PWD%/*}${0#..} ;;
          *)  SHELLFILE=$(type -P $0) ; if [ "${SHELLFILE:0:1}" != "/" ]; then SHELLFILE="${PWD}/$SHELLFILE" ; fi ;;
esac

# readlink -m makes sure SHELLDIR is a simple absolute path
SHELLDIR=`readlink -m "${SHELLFILE%/*}"`

# SHELLDIR should be $NISGS_ROOT/SPA/spa_directory, so...
SPA_HOME=${SHELLDIR}
SPA_DIR=${SHELLDIR}/..
NISGS_HOME=${SPA_DIR}/..

# The code below calls:
#
# tools/utilities/check_SPA_mode.sh
# 	which checks PROCESSING_MODE and makes station/ obey it
# tools/utilities/check_SPA_parallel.sh
#	which checks ADDITIONAL_SPA_INSTANCES and makes station/ obey it
#
# If both are needed, they must be called in that order

# Yell if we can't find check_SPA_mode.sh
if [[ ( ! -x $NISGS_HOME/tools/utilities/check_SPA_mode.sh ) ]]; then
    echo ERROR: could not find check_SPA_mode.sh
    exit -1
fi

# Switch SPA's mode
$NISGS_HOME/tools/utilities/check_SPA_mode.sh $SPA_HOME

# Uncomment the code below if this SPA should be run in parallel
#
# Yell if we can't find check_SPA_parallel.sh
if [[ ( ! -x $NISGS_HOME/tools/utilities/check_SPA_parallel.sh ) ]]; then
    echo ERROR: could not find check_SPA_parallel.sh
    exit -1
fi

# Apply parallelism if needed
$NISGS_HOME/tools/utilities/check_SPA_parallel.sh $SPA_HOME
